import java.sql.*;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.text.*;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;

/**
 * Dump out an JDBC-ResultSet by using
 * an ASCII-Table.
 * 
 * For using remark, the JDBC-ResultSet is
 * computed after creating dump.
 * 
 * Repeat the statement execution for
 * refreshing the JDBC-ResultSet.
 * 
 * <pre>
 * ResultSet rs = stmt.executeQuery(...);
 * if ( IS_DEBUG )
 * {
 *   ResultSetDumper.dump( rs );
 *   rs = stmt.executeQuery(...);
 * }
 * </pre>
 * 
 * @author hkuecker
 */
public class ResultSetDumper {

    /**
     * 
     * @param pResultSet
     * @return
     * @throws SQLException
     */
    public static String dump(//--
            final ResultSet pResultSet) //--
        throws SQLException //--
    {
        final ResultSetMetaData metaData = pResultSet.getMetaData();

        final int columnCount = metaData.getColumnCount();

        final ArrayList rowList = new ArrayList();
        while (pResultSet.next()){
            final Object[] colArr = new Object[ columnCount ];
            for (int i = 0; i < colArr.length; i++) {
                colArr[i] = pResultSet.getObject(i + 1);
            }
            rowList.add( colArr );
        }
        
        if (rowList.size() < 1){
        	return "ResultSet is empty, no rows selected";
        }

        AsciiTable.Cell[][] table = new AsciiTable.Cell[ rowList.size() + 1 ][ columnCount ];

        //Spalten-Namen
        for (int i = 0; i < columnCount; i++) {
            table[0][i] = new AsciiTable.Cell( metaData.getColumnName(i + 1) );
        }

        //Schleife ber die Zeilen des ResultSet
        for (int i = 0; i < rowList.size(); i++) {
            final Object[] colArr = (Object[]) rowList.get(i);
            //Schleife ber die Spalten einer Zeile des ResultSet
            for (int j = 0; j < columnCount; j++) {
                final String valueStr;
                final Object value = colArr[j];
                if (value instanceof Date){
                    //fix fr Problem 12-Stunden Anzeige auf Solaris-Rechner
                    valueStr = toFullStr( (Date) value );
                }
                else{
                    valueStr = "" + value;
                }
                    
				final AsciiTable.Cell cell = new AsciiTable.Cell( valueStr );
                table[ i + 1 ][ j ] = cell;

                if (value instanceof Number){
                    // Zahlen rechts ausrichten
                    cell.setAlignment( AsciiTable.RIGHT_ALIGNMENT );
                }
            }
        }
        table = AsciiTable.addCountCol(table, 1, 1, "lfd", 0, null);
        return "\n" + AsciiTable.buildLayout(table, AsciiTable.DOUBLE_LINE, AsciiTable.SINGLE_LINE, AsciiTable.DOUBLE_LINE);
    }

    /**
     * Formatierung {@link Date} oder
     * {@link java.sql.Timestamp}.
     * 
     * @param pObject
     * @return
     */
    public static String toFullStr(Date pDate) {
        if (pDate == null){
            return null;
        }
        if (pDate instanceof Timestamp){
            // Konvertieren Nanos in Millisekunden
            // siehe Javadoc of java.sql.Timestamp
            pDate = new Date( ((Timestamp) pDate ).getTime() 
                    + ( ((Timestamp) pDate ).getNanos() / 1000000 ) );
        }
        //Version for Jakarta Commons Fast Date Format
        //final FastDateFormat fastDateFormat = FastDateFormat.getInstance( "yyyy-MM-dd HH:mm:ss.SSS" );
        //final String dateStr = fastDateFormat.format( pDate );

        final DateFormat dateFormat = new SimpleDateFormat( "yyyy-MM-dd HH:mm:ss.SSS" );
        final String dateStr = dateFormat.format( pDate );

        return dateStr;
    }

}
